@@ -13,13 +13,13 @@ use tracing_subscriber::filter::LevelFilter;
 
 use pea2pea::{
     connect_nodes,
-    protocols::{Handshaking, Reading, Writing},
+    protocols::{Handshake, Reading, Writing},
     Connection, ConnectionSide, Node, Pea2Pea, Topology,
 };
 
 use std::{
     collections::HashMap,
-    io::{self, Read},
+    io,
     net::SocketAddr,
     sync::{
         atomic::{AtomicUsize, Ordering::Relaxed},
@@ -57,8 +57,7 @@ impl Player {
 
     fn throw_potato(&self) {
         let message = Message::IHaveThePotato(self.node().name().into());
-        let message = bincode::serialize(&message).unwrap();
-        self.node().send_broadcast(message.into());
+        self.send_broadcast(message);
 
         let (new_carrier_name, new_carrier_addr) = self
             .other_players
@@ -70,9 +69,7 @@ impl Player {
 
         info!(parent: self.node().span(), "throwing the potato to player {}!", new_carrier_name);
 
-        let message = bincode::serialize(&Message::HotPotato).unwrap();
-        self.node()
-            .send_direct_message(new_carrier_addr, message.into())
+        self.send_direct_message(new_carrier_addr, Message::HotPotato)
             .unwrap();
     }
 }
@@ -84,7 +81,7 @@ impl Pea2Pea for Player {
 }
 
 #[async_trait::async_trait]
-impl Handshaking for Player {
+impl Handshake for Player {
     async fn perform_handshake(&self, mut conn: Connection) -> io::Result<Connection> {
         let mut buffer = [0u8; 16];
 
@@ -123,7 +120,7 @@ impl Handshaking for Player {
     }
 }
 
-#[derive(Serialize, Deserialize)]
+#[derive(Serialize, Deserialize, Clone)]
 enum Message {
     HotPotato,
     IHaveThePotato(PlayerName),
@@ -139,27 +136,10 @@ impl Reading for Player {
         reader: &mut R,
     ) -> io::Result<Option<Self::Message>> {
         // expecting inbound messages to be prefixed with their length encoded as a LE u16
-        let mut len_arr = [0u8; 2];
-        if reader.read_exact(&mut len_arr).is_err() {
-            return Ok(None);
-        }
-        let payload_len = u16::from_le_bytes(len_arr) as usize;
+        let vec = common::read_len_prefixed_message::<R, 2>(reader)?;
 
-        if payload_len == 0 {
-            return Err(io::ErrorKind::InvalidData.into());
-        }
-
-        let mut buffer = vec![0u8; payload_len];
-        if reader
-            .take(payload_len as u64)
-            .read_exact(&mut buffer)
-            .is_err()
-        {
-            Ok(None)
-        } else {
-            let message = bincode::deserialize(&buffer).unwrap();
-            Ok(Some(message))
-        }
+        vec.map(|v| bincode::deserialize(&v).map_err(|_| io::ErrorKind::InvalidData.into()))
+            .transpose()
     }
 
     async fn process_message(&self, _source: SocketAddr, message: Self::Message) -> io::Result<()> {
@@ -195,14 +175,18 @@ impl Reading for Player {
 }
 
 impl Writing for Player {
+    type Message = Message;
+
     fn write_message<W: io::Write>(
         &self,
         _: SocketAddr,
-        payload: &[u8],
+        payload: &Self::Message,
         writer: &mut W,
     ) -> io::Result<()> {
-        writer.write_all(&(payload.len() as u16).to_le_bytes())?;
-        writer.write_all(payload)
+        let payload_len = bincode::serialized_size(payload).unwrap() as u16;
+        writer.write_all(&payload_len.to_le_bytes())?;
+        bincode::serialize_into(writer, payload)
+            .map_err(|e| io::Error::new(io::ErrorKind::Other, e))
     }
 }
 
@@ -224,7 +208,7 @@ async fn main() {
     }
 
     for player in &players {
-        player.enable_handshaking();
+        player.enable_handshake();
         player.enable_reading();
         player.enable_writing();
     }
