@@ -17,7 +17,7 @@ static RANDOM_BYTES: Lazy<Bytes> = Lazy::new(|| {
     Bytes::from(
         (&mut SmallRng::from_entropy())
             .sample_iter(Standard)
-            .take(64 * 1024 - 4)
+            .take(MSG_SIZE - 4)
             .collect::<Vec<_>>(),
     )
 });
@@ -120,7 +120,6 @@ async fn run_bench_scenario(sender_count: usize) -> f64 {
         tokio::spawn(async move {
             for _ in 0..NUM_MESSAGES {
                 spammer
-                    .node()
                     .send_direct_message(sink_addr, RANDOM_BYTES.clone())
                     .unwrap();
             }
